@@ -2,8 +2,10 @@ import traceback
 from abc import ABC, abstractmethod
 from datetime import datetime
 import logging
+from jaaql.utilities.utils_no_project_imports import time_delta_ms
 from jaaql.exceptions.http_status_exception import *
 from typing import Optional
+import queue
 
 ERR__unknown_echo = "Unknown echo type '%s'. Please use either %s"
 
@@ -102,8 +104,9 @@ class DBInterface(ABC):
         self.__attempt_commit_rollback(conn, err)
         self.__err_to_exception(err, echo)
 
-    def put_conn_handle_error(self, conn, err, echo=ECHO__none):
-        self.__attempt_commit_rollback(conn, err)
+    def put_conn_handle_error(self, conn, err, echo=ECHO__none, skip_rollback_commit: bool = False):
+        if not skip_rollback_commit:
+            self.__attempt_commit_rollback(conn, err)
 
         try:
             self.put_conn(conn)
@@ -120,7 +123,7 @@ class DBInterface(ABC):
     def rollback(self, conn):
         pass
 
-    def execute_query_fetching_results(self, conn, query, parameters=None, echo=ECHO__none, as_objects=False):
+    def execute_query_fetching_results(self, conn, query, parameters=None, echo=ECHO__none, as_objects=False, wait_hook: queue.Queue = None):
         if echo not in ECHO__allowed:
             allowed_echoes = ", ".join([str(allowed_echo) for allowed_echo in ECHO__allowed])
             raise HttpStatusException(ERR__unknown_echo % (str(echo), allowed_echoes), HTTPStatus.BAD_REQUEST)
@@ -139,7 +142,7 @@ class DBInterface(ABC):
                     else:
                         new_parameters[key] = value
 
-            columns, rows = self.execute_query(conn, query, new_parameters)
+            columns, rows = self.execute_query(conn, query, new_parameters, wait_hook)
 
             ret = {
                 RET__columns: columns,
@@ -195,7 +198,7 @@ class DBInterface(ABC):
             return ret, err
 
     @abstractmethod
-    def execute_query(self, conn, query, parameters: Optional[dict] = None):
+    def execute_query(self, conn, query, parameters: Optional[dict] = None, wait_hook: queue.Queue = None):
         pass
 
     @abstractmethod
