@@ -1,7 +1,7 @@
 from jaaql.openapi.swagger_documentation import *
 from jaaql.constants import *
 from jaaql.documentation.documentation_shared import ARG_RES__username, EXAMPLE__password, ARG_RES__password, RES__oauth_token, ARG_RES__tenant,\
-    rename_arg, set_required
+    rename_arg, set_required, set_nullable
 
 TITLE = "JAAQL API"
 DESCRIPTION = "Collection of methods in the JAAQL API"
@@ -72,7 +72,7 @@ DOCUMENTATION__create_tenant_account = SwaggerDocumentation(
 )
 
 ARG_RES__email_template = SwaggerArgumentResponse(
-    name=KEY__email_template_name,
+    name=KEY__email_template,
     description="Email template name",
     arg_type=str,
     example=["signup"]
@@ -118,6 +118,7 @@ DOCUMENTATION__sign_up_request_invite = SwaggerDocumentation(
             ARG_RES__parameters,
             ARG_RES__tenant,
             ARG_RES__configuration,
+            ARG_RES__application,
             ARG_RES__email_template,
             ARG_RES__already_signed_up_email_template
         ],
@@ -245,6 +246,7 @@ DOCUMENTATION__reset_password = SwaggerDocumentation(
             ARG_RES__email,
             ARG_RES__tenant,
             ARG_RES__reset_password_email_template,
+            ARG_RES__application,
             ARG_RES__configuration,
             ARG_RES__parameters
         ],
@@ -344,6 +346,7 @@ DOCUMENTATION__password = SwaggerDocumentation(
 
 DOCUMENTATION__public_user = SwaggerDocumentation(
     tags="Public Users",
+    security=False,
     methods=SwaggerMethod(
         method=REST__GET,
         name="Fetch public user",
@@ -367,18 +370,8 @@ DOCUMENTATION__submit = SwaggerDocumentation(
         description="Executes a JAAQL query which is either a single SQL query or a list of queries. Returns results",
         method=REST__POST,
         arguments=ARG_RESP__allow_all,
-        response=RES__allow_all
-    )
-)
-
-DOCUMENTATION__submit_file = SwaggerDocumentation(
-    tags="Queries",
-    methods=SwaggerMethod(
-        name="Submits a file for execution",
-        description="Submits a full script file",
-        method=REST__POST,
-        arguments=ARG_RESP__allow_all,
-        response=RES__allow_all
+        response=RES__allow_all,
+        parallel_verification=True
     )
 )
 
@@ -393,7 +386,9 @@ ARG_RES__renderable_document = [
         name=KEY__parameters,
         description="Any parameters to pass to the url as http GET parameters",
         arg_type=ARG_RESP__allow_all
-    )
+    ),
+    set_nullable(ARG_RES__application, "Not required if being sent in an email as this is specified in the email"),
+    set_nullable(ARG_RES__configuration, "Not required if being sent in an email as this is specified in the email"),
 ]
 
 ARG_RES__attachments_for_send = SwaggerArgumentResponse(
@@ -515,3 +510,33 @@ DOCUMENTATION__rendered_document = SwaggerDocumentation(
         )
     )
 )
+
+DOCUMENTATION__drop_email_account = SwaggerDocumentation(
+    tags="Emails",
+    methods=SwaggerMethod(
+        name="Drop email account",
+        description="Drops the email account",
+        method=REST__DELETE,
+        arguments=SwaggerArgumentResponse(
+            name=KEY__name,
+            description="The name of the email account",
+            arg_type=str,
+            example=["noreply"]
+        )
+    )
+)
+
+DOCUMENTATION__drop_databases = SwaggerDocumentation(
+    tags="Database",
+    methods=SwaggerMethod(
+        name="Drop database",
+        description="Drops the database associated with a tenant",
+        method=REST__DELETE,
+        arguments=SwaggerArgumentResponse(
+            name=KEY__name,
+            description="The name of the database",
+            arg_type=str,
+            example=["invoicing_live"]
+        )
+    )
+)
