@@ -6,7 +6,8 @@ import pkgutil
 from jaaql.openapi.swagger_documentation import produce_all_documentation
 from jaaql.email.email_manager_service import create_flask_app as create_email_service_app
 from jaaql.services.script_install import bootup
-from jaaql.utilities.options import *
+from typing import List
+from jaaql.utilities.options import parse_options, OPT_KEY__vault_key, OPT_KEY__email_credentials, OPT_KEY__local_install, Option, OPT_KEY__profiling
 import sys
 import threading
 import os
@@ -51,7 +52,7 @@ def dir_non_builtins(folder):
 
 
 def create_app(is_gunicorn: bool = False, override_config_path: str = None, migration_db_interface=None,
-               migration_project_name: str = None, migration_folder: str = None, supplied_documentation = None,
+               migration_project_name: str = None, migration_folder: str = None, supplied_documentation=None,
                controllers: [JAAQLControllerInterface] = None, models: [JAAQLModelInterface] = None, additional_options: List[Option] = None,
                start_email_service: bool = False, **options):
     if controllers is None:
@@ -71,9 +72,9 @@ def create_app(is_gunicorn: bool = False, override_config_path: str = None, migr
     if len(options) == 0 and not is_gunicorn:
         options = parse_options(sys.argv, False, additional_options=additional_options)
 
-    # if not is_gunicorn and start_email_service:
-    #     threading.Thread(target=create_email_service_app, args=[options.get(OPT_KEY__vault_key), options.get(OPT_KEY__email_credentials)],
-    #                      daemon=True).start()
+    if not is_gunicorn and start_email_service:
+        threading.Thread(target=create_email_service_app, args=[options.get(OPT_KEY__vault_key), options.get(OPT_KEY__email_credentials)],
+                         daemon=True).start()
     if not is_gunicorn:
         threading.Thread(target=bootup, args=[options.get(OPT_KEY__vault_key), False, OPT_KEY__local_install in options], daemon=True).start()
 
