@@ -33,6 +33,7 @@ ARG__tenant = "tenant"
 ARG__auth_token = "auth_token"
 ARG__connection = "connection"
 ARG__is_public = "is_public"
+ARG__verification_hook = "verification_hook"
 
 CONTENT__encoding = "charset=utf-8"
 CONTENT__json = "application/json"
@@ -397,9 +398,9 @@ class BaseJAAQLController:
         def wrap_func(view_func):
             @wraps(view_func)
             def routed_function(view_func_local):
+                start_time = datetime.now()
                 request_id = uuid.uuid4()
                 self.perform_profile(request_id, route=route, method=request.method)
-                start_time = datetime.now()
                 resp = None
                 resp_type = current_app.config["JSONIFY_MIMETYPE"]
                 jaaql_resp = JAAQLResponse()
@@ -414,12 +415,20 @@ class BaseJAAQLController:
                     tenant = None
                     username = None
                     is_public = None
+                    verification_hook = None
+                    if method.parallel_verification:
+                        verification_hook = Queue()
 
                     ip_addr = request.headers.get(HEADER__real_ip, request.remote_addr).split(",")[0]
 
                     if swagger_documentation.security:
-                        user_id, tenant, username, ip_id, is_public = self.model.verify_auth_token(request.headers.get(HEADER__security), ip_addr)
-                        self.perform_profile(request_id, "Verify JWT")
+                        if verification_hook:
+                            user_id, tenant, username, ip_id, is_public = self.model.verify_auth_token_threaded(request.headers.get(HEADER__security),
+                                                                                                                ip_addr, verification_hook)
+                            self.perform_profile(request_id, "Verify JWT Threaded")
+                        else:
+                            user_id, tenant, username, ip_id, is_public = self.model.verify_auth_token(request.headers.get(HEADER__security), ip_addr)
+                            self.perform_profile(request_id, "Verify JWT")
 
                     supply_dict = {}
 
@@ -436,6 +445,9 @@ class BaseJAAQLController:
                                 raise Exception(ERR__method_required_user_id)
                             supply_dict[ARG__user_id] = user_id
 
+                        if method.parallel_verification:
+                            supply_dict[ARG__verification_hook] = verification_hook
+
                         if ARG__tenant in inspect.getfullargspec(view_func_local).args:
                             if not swagger_documentation.security:
                                 raise Exception(ERR__method_required_tenant)
