@@ -5,6 +5,7 @@ from jaaql.documentation.documentation_public import *
 from jaaql.documentation.documentation_shared import *
 from jaaql.mvc.response import JAAQLResponse
 from jaaql.db.db_interface import DBInterface
+import queue
 
 
 class JAAQLController(BaseJAAQLController):
@@ -46,9 +47,17 @@ class JAAQLController(BaseJAAQLController):
         def is_alive():
             self.model.is_alive()
 
+        @self.cors_route('/email/account', DOCUMENTATION__drop_email_account)
+        def drop_email_account(tenant: str, http_inputs: dict, connection: DBInterface):
+            self.model.drop_email_account(tenant, connection, **http_inputs)
+
+        @self.cors_route('/database', DOCUMENTATION__drop_databases)
+        def drop_database(tenant: str, http_inputs: dict, connection: DBInterface):
+            self.model.drop_database(tenant, connection, **http_inputs)
+
         @self.cors_route('/accounts', DOCUMENTATION__create_tenant_account)
         def accounts(connection: DBInterface, tenant: str, http_inputs: dict):
-            self.model.create_tenant_account(connection, tenant, **http_inputs)
+            self.model.create_tenant_account(connection, tenant, http_inputs)
 
         @self.cors_route('/account/password', DOCUMENTATION__password)
         def password(user_id: str, username: str, tenant: str, ip_address: str, http_inputs: dict):
@@ -59,12 +68,8 @@ class JAAQLController(BaseJAAQLController):
             return self.model.fetch_public_user(**http_inputs)
 
         @self.cors_route('/submit', DOCUMENTATION__submit)
-        def submit(http_inputs: dict, user_id: str, tenant: str):
-            return self.model.submit(http_inputs, user_id, tenant)
-
-        @self.cors_route('/submit-file', DOCUMENTATION__submit_file)
-        def submit_file(http_inputs: dict, user_id: str, tenant: str):
-            return self.model.submit(http_inputs, user_id, tenant, force_transactional=True)
+        def submit(http_inputs: dict, user_id: str, tenant: str, verification_hook: queue.Queue):
+            return self.model.submit(http_inputs, user_id, tenant, verification_hook=verification_hook)
 
         @self.cors_route('/account/signup/request', DOCUMENTATION__sign_up_request_invite)
         def signup_request(http_inputs: dict):
@@ -95,22 +100,22 @@ class JAAQLController(BaseJAAQLController):
             return self.model.reset_password_status(http_inputs)
 
         @self.cors_route('/account/reset-password/reset', DOCUMENTATION__reset_password_with_invite)
-        def reset_password_reset(http_inputs: dict, tenant: str):
-            return self.model.reset_password_perform_reset(http_inputs, tenant)
+        def reset_password_reset(http_inputs: dict):
+            return self.model.reset_password_perform_reset(http_inputs)
 
         @self.cors_route(ENDPOINT__jaaql_emails, DOCUMENTATION__email)
-        def emails(http_inputs: dict, tenant: str, oauth_token: str, user_id: str, ip_address: str, is_public: bool):
+        def emails(http_inputs: dict, tenant: str, auth_token: str, user_id: str, ip_address: str, is_public: bool, username: str):
             if is_public:
                 raise HttpStatusException(ERR__user_public, HTTPStatus.UNAUTHORIZED)
 
-            return self.model.send_email(http_inputs, oauth_token, user_id, tenant, ip_address)
+            return self.model.send_email(http_inputs, auth_token, user_id, tenant, ip_address, username)
 
         @self.cors_route('/documents', DOCUMENTATION__document)
-        def documents(http_inputs: dict, oauth_token: str, ip_address: str, tenant: str, response: JAAQLResponse):
+        def documents(http_inputs: dict, auth_token: str, ip_address: str, tenant: str, response: JAAQLResponse):
             if self.is_get():
                 return self.model.fetch_document(http_inputs, response)
             else:
-                return self.model.render_document(http_inputs, oauth_token, ip_address, tenant)
+                return self.model.render_document(http_inputs, auth_token, ip_address, tenant)
 
         @self.cors_route('/rendered_documents', DOCUMENTATION__rendered_document)
         def documents(http_inputs: dict):
