@@ -8,7 +8,8 @@ from jaaql.db.db_interface import DBInterface, RET__rows
 from jaaql.interpreter.interpret_jaaql import InterpretJAAQL
 from jaaql.mvc.queries import *
 from jaaql.utilities.utils import get_jaaql_root
-from jaaql.db.db_utils import execute_supplied_statement, execute_supplied_statement_singleton, create_interface, jaaql__encrypt
+from jaaql.db.db_utils import execute_supplied_statement, execute_supplied_statement_singleton, create_interface, jaaql__encrypt,\
+    load_tenant_symmetric_keys
 from jaaql.utilities import crypt_utils
 from io import BytesIO
 from flask import send_file
@@ -16,11 +17,12 @@ import threading
 from datetime import datetime, timedelta
 from jaaql.mvc.base_model import JAAQLPivotInfo
 from os.path import dirname
+from jwt.utils import base64url_decode
 import uuid
 import json
 import random
-
 import os
+from queue import Queue
 
 ERR__refresh_expired = "Token too old to be used for refresh. Please authenticate again"
 ERR__incorrect_install_key = "Incorrect install key!"
@@ -29,7 +31,6 @@ ERR__denied_create_user = "Cannot create that user with your current permissions
 ERR__incorrect_credentials = "Incorrect credentials!"
 ERR__lacking_permissions = "Only an administrator can perform this action!"
 ERR__short_tenant_name = "Tenant name is too short!"
-ERR__invalid_token = "Invalid token!"
 ERR__schema_invalid = "Schema invalid!"
 ERR__cant_send_attachments = "Cannot send attachments to other people"
 ERR__keep_alive_failed = "Keep alive failed"
@@ -89,22 +90,62 @@ SIGNUP__completed = 3
 
 class JAAQLModel(BaseJAAQLModel):
 
+    VERIFICATION_QUEUE = None
+    LOG_QUEUE = None
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        JAAQLModel.LOG_QUEUE = Queue()
+        threading.Thread(target=self.log_thread, args=[JAAQLModel.LOG_QUEUE])
+        self.tenant_reload_lock = threading.Lock()
+        self.tenant_symmetric_keys = None
+        if not self.is_container:
+            self.load_keys()
+
+    def load_keys(self):
+        if self.vault.has_obj(VAULT_KEY__jaaql_lookup_connection):
+            self.tenant_symmetric_keys = load_tenant_symmetric_keys(self.jaaql_lookup_connection, self.get_db_crypt_key())
+
+    def fetch_tenant_symmetric_key(self, tenant: str):
+        if tenant in self.tenant_symmetric_keys:
+            return self.tenant_symmetric_keys[tenant]
+
+        with self.tenant_reload_lock:
+            if tenant not in self.tenant_symmetric_keys:
+                self.load_keys()
+
+        return self.tenant_symmetric_keys[tenant]
 
     def check_is_internal_admin(self, connection: DBInterface):
         execute_supplied_statement(connection, QUERY__fetch_singleton_as_permissions_check)
 
-    def log(self, user_id: str, occurred: datetime, duration_ms: int, exception: str, contr_input: str, ip: str, status: int,
-            endpoint: str, databases: list = None):
-        pass  # TODO use a log threading pool
+    def log(self, user_id: str, occurred: datetime, duration_ms: int, exception: str, contr_input: str, ip: str, status: int, endpoint: str):
+        JAAQLModel.LOG_QUEUE.put({
+            KEY__user_id: user_id,
+            KEY__occurred: occurred.isoformat(),
+            KEY__duration_ms: duration_ms,
+            KEY__exception: exception,
+            KEY__input: contr_input,
+            KEY__ip: ip,
+            KEY__status: int(status),
+            KEY__endpoint: endpoint
+        })
 
-    def log_sync(self, user_id: str, occurred: datetime, duration_ms: int, exception: str, contr_input: str, ip: str, status: int, endpoint: str,
-                 databases: list = None):
-        pass  # TODO
+    def log_thread(self, log_queue: Queue):
+        while True:
+            execute_supplied_statement(self.jaaql_lookup_connection, QUERY__log_ins, log_queue.get())
 
-    def drop_email_account(self):
-        pass  # TODO change how we are doing email accounts, end to end signup request with data in a new database, end to end reset password
+    def check_admin(self, connection: DBInterface, option: str):
+        execute_supplied_statement(connection, "SELECT * FROM check_" + option)  # Will error if this fails
+
+    def drop_email_account(self, tenant: str, connection: DBInterface, name: str):
+        self.check_admin(connection, "drop_email_account")
+        execute_supplied_statement(self.jaaql_lookup_connection, QUERY__drop_email_account, {KEY__name: name, KEY__tenant: tenant})
+        self.email_manager.reload_service()
+
+    def drop_database(self, tenant: str, connection: DBInterface, name: str):
+        self.check_admin(connection, "drop_database")
+        execute_supplied_statement(self.jaaql_lookup_connection, QUERY__drop_database, {KEY__name: name, KEY__tenant: tenant})
 
     def redeploy(self, connection: DBInterface):
         self.check_is_internal_admin(connection)
@@ -121,11 +162,14 @@ class JAAQLModel(BaseJAAQLModel):
         threading.Thread(target=self.exit_jaaql, daemon=True).start()
 
     def add_tenant(self, connection: DBInterface, name: str, tenant_admin_password: str = None, attach_tenant_admin_as: str = None):
-        name = name.lower().strip()
         if len(name) < 3:
             raise HttpStatusException(ERR__short_tenant_name)
 
-        tenant_admin_account_id = execute_supplied_statement_singleton(connection, QUERY__create_tenant, {KEY__the_tenant: name})[RET__rows][0]
+        _, sk = crypt_utils.key_stretcher(str(uuid.uuid4()), length=crypt_utils.AES__key_length)
+        sk = sk.decode()
+
+        tenant_admin_account_id = execute_supplied_statement_singleton(connection, QUERY__create_tenant,
+                                                                       {KEY__the_tenant: name, KEY__enc_symmetric_key: sk})[RET__rows][0]
 
         execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__attach_account, {
             KEY__the_tenant: name,
@@ -133,14 +177,17 @@ class JAAQLModel(BaseJAAQLModel):
             KEY__enc_username: attach_tenant_admin_as if attach_tenant_admin_as else USERNAME__super
         }, encryption_key=self.get_db_crypt_key(), encrypt_parameters=[KEY__enc_username], encryption_salts=self.get_repeatable_salt(name))
         self.add_account_password(tenant_admin_account_id, tenant_admin_password)
+        self.fetch_tenant_symmetric_key(name)
 
     def add_account_password(self, user_id: str, password: str):
         crypt_utils.validate_password(password)
         salt = self.get_repeatable_salt(user_id)
         password = crypt_utils.hash_password(password, salt)
-        execute_supplied_statement(self.jaaql_lookup_connection, QUERY__add_password, {KEY__account: user_id, KEY__password_hash: password},
-                                   encryption_key=self.get_db_crypt_key(), encryption_salts={KEY__password_hash: salt},
-                                   encrypt_parameters=[KEY__password_hash])
+        password_id = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__add_password,
+                                                           {KEY__account: user_id, KEY__password_hash: password},
+                                                           encryption_key=self.get_db_crypt_key(), encryption_salts={KEY__password_hash: salt},
+                                                           encrypt_parameters=[KEY__password_hash], as_objects=True)[KEY__id]
+        execute_supplied_statement(self.jaaql_lookup_connection, QUERY__add_password_cache, {KEY__account: user_id, KEY__password: password_id})
 
     def add_my_account_password(self, user_id: str, username: str, tenant: str, ip_address: str, old_password: str, password: str):
         res = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_recent_passwords_with_account, {KEY__user_id: user_id},
@@ -153,7 +200,7 @@ class JAAQLModel(BaseJAAQLModel):
     def fetch_user_from_username(self, username: str, tenant: str):
         return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_user_from_username, {KEY__username: username},
                                                     encryption_key=self.get_db_crypt_key(), encrypt_parameters=[KEY__username],
-                                                    encryption_salts={KEY__username: self.get_repeatable_salt(tenant)}, as_objects=True)[KEY__id]
+                                                    encryption_salts={KEY__username: self.get_repeatable_salt(tenant)}, as_objects=True)[KEY__user_id]
 
     def fetch_user_record_from_username(self, username: str, tenant: str):
         return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_user_record_from_username, {KEY__username: username},
@@ -191,15 +238,18 @@ class JAAQLModel(BaseJAAQLModel):
         select_table = template[KEY__data_validation_view] if template[KEY__data_validation_view] is not None else val_table
         return self.select_from_data_validation_table(connection, select_table, pkey_vals), pkey_vals
 
-    def user_invite(self, user_id: str, configuration: str = None, email_template: str = None, template_lookup: dict = None):
+    def user_invite(self, user_id: str, tenant: str = None, application: str = None, configuration: str = None, email_template: str = None,
+                    template_lookup: dict = None):
         if template_lookup is not None:
             template_lookup = json.dumps(template_lookup)
 
         params = {
             ATTR__data_lookup_json: template_lookup,
             ATTR__the_user: user_id,
+            KEY__application: application,
             KEY__configuration: configuration,
             KEY__email_template: email_template,
+            KEY__tenant: tenant,
             KEY__invite_code: "".join([CODE__letters[random.randint(0, len(CODE__letters) - 1)] for _ in range(CODE__invite_length)])
         }
         return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_insert, params, as_objects=True)
@@ -207,22 +257,25 @@ class JAAQLModel(BaseJAAQLModel):
     def fetch_interface_for_tenant(self, tenant: str, application: str, configuration: str, schema: str):
         schemas = self.fetch_pivoted_application_schemas(tenant)
         db_name = schemas[application][configuration][schema][KEY__database]
-        return self.create_interface_for_db(ROLE__jaaql, tenant + "__" + db_name)
+        return self.create_interface_for_db(ROLE__jaaql if tenant == TENANT__jaaql else tenant + "__admin", tenant + "__" + db_name)
 
     def request_signup(self, inputs: dict):
         template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url, {
-            KEY__name: inputs[KEY__email_template_name],
+            KEY__name: inputs[KEY__email_template],
             KEY__tenant: inputs[KEY__tenant],
-            KEY__configuration: inputs[KEY__configuration]
+            KEY__configuration: inputs[KEY__configuration],
+            KEY__application: inputs[KEY__application]
         }, as_objects=True)
         template_already_exists = execute_supplied_statement_singleton(
             self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url, {
                 KEY__name: inputs[KEY__already_signed_up_email_template],
                 KEY__tenant: inputs[KEY__tenant],
-                KEY__configuration: inputs[KEY__configuration]
+                KEY__configuration: inputs[KEY__configuration],
+                KEY__application: inputs[KEY__application]
             }, as_objects=True
         )
-        app_url = template.pop(KEY__url)
+        artifact_url = template.pop(KEY__artifact_base_uri)
+        app_url = template.pop(KEY__application_url)
 
         if not template[KEY__allow_signup] or not template_already_exists[KEY__allow_confirm_signup_attempt]:
             raise HttpStatusException(ERR__template_not_signup)
@@ -234,8 +287,12 @@ class JAAQLModel(BaseJAAQLModel):
             if not hs.message.startswith(SQL__err_duplicate_user):
                 raise hs  # Unrelated exception, raise it
 
-            user_id = self.fetch_user_from_username(inputs[KEY__email], inputs[KEY__tenant])
-            user_existed = True
+            res = self.fetch_user_recent_password(inputs[KEY__email], inputs[KEY__tenant])
+            if len(res) == 0:
+                user_id = self.fetch_user_from_username(inputs[KEY__email], inputs[KEY__tenant])
+            else:
+                user_id = res[0][KEY__user_id]
+                user_existed = True
 
         attempts = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_count, {KEY__user_id: user_id},
                                                         as_objects=True)[ATTR__count]
@@ -257,16 +314,21 @@ class JAAQLModel(BaseJAAQLModel):
                 raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__signup_key)
             if EMAIL_PARAM__invite_code in params:
                 raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__invite_code)
+            if EMAIL_PARAM__app_url in params:
+                raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__app_url)
 
             interface = self.fetch_interface_for_tenant(inputs[KEY__tenant], template[KEY__application], inputs[KEY__configuration],
                                                         template[KEY__schema])
             sanitized_params, pkey_vals = self.fetch_sanitized_email_params(interface, template, params)
 
-        invite_keys = self.user_invite(user_id, inputs[KEY__configuration], inputs[KEY__email_template_name], pkey_vals)
-        optional_params = {EMAIL_PARAM__signup_key: invite_keys[KEY__invite_key], EMAIL_PARAM__invite_code: invite_keys[KEY__invite_code]}
+        invite_keys = self.user_invite(user_id, inputs[KEY__tenant], inputs[KEY__application], inputs[KEY__configuration], template[KEY__name],
+                                       pkey_vals)
+        optional_params = {EMAIL_PARAM__signup_key: invite_keys[KEY__invite_key], EMAIL_PARAM__invite_code: invite_keys[KEY__invite_code],
+                           EMAIL_PARAM__app_url: app_url}
         optional_params = {**optional_params, **sanitized_params}
 
-        self.email_manager.construct_and_send_email(self.url, app_url, template, user_id, inputs[KEY__email], None, {}, optional_params)
+        self.email_manager.construct_and_send_email(artifact_url, template, user_id, inputs[KEY__tenant], inputs[KEY__email], None, {},
+                                                    optional_params)
 
         return {KEY__invite_key: invite_keys[KEY__invite_poll_key]}
 
@@ -276,13 +338,8 @@ class JAAQLModel(BaseJAAQLModel):
                                                     decrypt_columns=[KEY__email], encryption_key=self.get_db_crypt_key(),
                                                     singleton_message=ERR__cant_find_sign_up)
 
-        existed = False
-        try:
-            self.fetch_user_from_username(resp[KEY__email], self.jaaql_lookup_connection)
-            existed = True
-        except HttpStatusException as sub_hs:
-            if sub_hs.response_code != HTTPStatus.UNAUTHORIZED:
-                raise sub_hs  # Unrelated exception, raise it
+        res = self.fetch_user_recent_password(resp[KEY__email], resp[KEY__tenant])
+        existed = len(res) != 0
 
         is_invite_key = str(resp[KEY__invite_key]) == inputs[KEY__invite_or_poll_key]
 
@@ -316,6 +373,13 @@ class JAAQLModel(BaseJAAQLModel):
 
         return {KEY__invite_key_status: status}
 
+    def fetch_user_recent_password(self, username: str, tenant: str):
+        res = execute_supplied_statement(self.jaaql_lookup_connection, QUERY__fetch_recent_passwords, {
+            KEY__enc_username: username
+        }, encryption_salts={KEY__enc_username: self.get_repeatable_salt(tenant)}, decrypt_columns=[KEY__password_hash, KEY__username],
+                                         encrypt_parameters=[KEY__enc_username], encryption_key=self.get_db_crypt_key(), as_objects=True)
+        return res
+
     def sign_up_user_with_token(self, token: str, password: str):
         resp = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__sign_up_fetch, parameters={KEY__invite_or_poll_key: token},
                                                     decrypt_columns=[KEY__email], encryption_key=self.get_db_crypt_key(),
@@ -323,15 +387,12 @@ class JAAQLModel(BaseJAAQLModel):
 
         username = resp[KEY__email]
 
-        res = execute_supplied_statement(self.jaaql_lookup_connection, QUERY__fetch_recent_passwords, {
-            KEY__enc_username: username
-        }, encryption_salts={KEY__enc_username: self.get_repeatable_salt(resp[KEY__tenant])}, decrypt_columns=[KEY__password_hash, KEY__username],
-                                         encrypt_parameters=[KEY__enc_username], encryption_key=self.get_db_crypt_key(), as_objects=True)
+        res = self.fetch_user_recent_password(username, resp[KEY__tenant])
 
         if len(res) != 0:
             raise HttpStatusException(ERR__already_signed_up, response_code=HTTPStatus.CONFLICT)
 
-        self.add_account_password(res[KEY__user_id], password)
+        self.add_account_password(self.fetch_user_from_username(username, resp[KEY__tenant]), password)
 
         return {KEY__email: username}
 
@@ -344,8 +405,9 @@ class JAAQLModel(BaseJAAQLModel):
         if resp[KEY__email_template] and resp[ATTR__data_lookup_json]:
             template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url,
                                                             {KEY__name: resp[KEY__email_template], KEY__tenant: tenant,
-                                                             KEY__configuration: resp[KEY__configuration]}, as_objects=True)
-            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], resp[KEY__schema], template[KEY__schema])
+                                                             KEY__configuration: resp[KEY__configuration], KEY__application: resp[KEY__application]},
+                                                            as_objects=True)
+            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], resp[KEY__configuration], template[KEY__schema])
             res[KEY__parameters] = self.select_from_data_validation_table(interface, template[KEY__data_validation_table],
                                                                           json.loads(resp[ATTR__data_lookup_json]))
 
@@ -368,7 +430,7 @@ class JAAQLModel(BaseJAAQLModel):
                                                             {KEY__name: resp[KEY__email_template], KEY__tenant: tenant,
                                                              KEY__configuration: resp[KEY__configuration]},
                                                             as_objects=True)[KEY__data_validation_table]
-            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], resp[KEY__schema], template[KEY__schema])
+            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], resp[KEY__configuration], template[KEY__schema])
             self.delete_from_data_validation_table(interface, template, json.loads(resp[ATTR__data_lookup_json]))
 
         execute_supplied_statement(self.jaaql_lookup_connection, QUERY__sign_up_close, {KEY__invite_key: resp[KEY__invite_key]})
@@ -377,9 +439,11 @@ class JAAQLModel(BaseJAAQLModel):
         template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url, {
             KEY__name: inputs[KEY__email_template],
             KEY__tenant: inputs[KEY__tenant],
-            KEY__configuration: inputs[KEY__configuration]
+            KEY__configuration: inputs[KEY__configuration],
+            KEY__application: inputs[KEY__application]
         }, as_objects=True)
-        app_url = template.pop(KEY__url)
+        app_url = template.pop(KEY__application_url)
+        artifact_url = template.pop(KEY__artifact_base_uri)
 
         if not template[KEY__allow_reset_password]:
             raise HttpStatusException(ERR__template_reset_password)
@@ -421,6 +485,8 @@ class JAAQLModel(BaseJAAQLModel):
                     raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__reset_key)
                 if EMAIL_PARAM__reset_code in params:
                     raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__reset_code)
+                if EMAIL_PARAM__app_url in params:
+                    raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__app_url)
 
                 interface = self.fetch_interface_for_tenant(inputs[KEY__tenant], template[KEY__application], inputs[KEY__configuration],
                                                             template[KEY__schema])
@@ -433,14 +499,18 @@ class JAAQLModel(BaseJAAQLModel):
                 ATTR__data_lookup_json: template_lookup,
                 ATTR__the_user: user[KEY__user_id],
                 KEY__email_template: template[KEY__name],
+                KEY__configuration: inputs[KEY__configuration],
+                KEY__application: inputs[KEY__application],
+                KEY__tenant: inputs[KEY__tenant],
                 KEY__reset_code: "".join([CODE__alphanumeric[random.randint(0, len(CODE__alphanumeric) - 1)] for _ in range(CODE__reset_length)])
             }
 
             reset_keys = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__reset_insert, params, as_objects=True)
-            optional_params = {EMAIL_PARAM__reset_key: reset_keys[KEY__reset_key], EMAIL_PARAM__reset_code: reset_keys[KEY__reset_code]}
+            optional_params = {EMAIL_PARAM__reset_key: reset_keys[KEY__reset_key], EMAIL_PARAM__reset_code: reset_keys[KEY__reset_code],
+                               EMAIL_PARAM__app_url: app_url}
             optional_params = {**optional_params, **sanitized_params}
 
-            self.email_manager.construct_and_send_email(self.url, app_url, template, user[KEY__user_id], inputs[KEY__email], None, {},
+            self.email_manager.construct_and_send_email(artifact_url, template, user[KEY__user_id], inputs[KEY__tenant], inputs[KEY__email], None, {},
                                                         optional_params)
         else:
             params = {
@@ -544,14 +614,14 @@ class JAAQLModel(BaseJAAQLModel):
 
             return send_file(buffer, as_attachment=as_attachment, download_name=res[KEY__filename])
 
-    def render_document(self, inputs: dict, oauth_token: str, ip_address: str, tenant: str):
-        inputs[KEY__oauth_token] = self.refresh_auth_token(oauth_token, ip_address)
+    def render_document(self, inputs: dict, auth_token: str, ip_address: str, tenant: str):
+        inputs[KEY__oauth_token] = self.refresh_auth_token(auth_token, ip_address)
         inputs[KEY__tenant] = tenant
         return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__ins_rendered_document, inputs,
                                                     encryption_key=self.get_db_crypt_key(), encrypt_parameters=[KEY__parameters, KEY__oauth_token],
                                                     as_objects=True)
 
-    def send_email(self, inputs: dict, oauth_token: str, user_id: str, tenant: str, ip_address: str):
+    def send_email(self, inputs: dict, auth_token: str, user_id: str, tenant: str, ip_address: str, username: str):
         if inputs[KEY__recipient] is not None:
             raise HttpStatusException(HTTPStatus.NOT_IMPLEMENTED.description, HTTPStatus.NOT_IMPLEMENTED.value)
 
@@ -560,9 +630,12 @@ class JAAQLModel(BaseJAAQLModel):
 
         template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url, {
             KEY__name: inputs[KEY__email_template],
-            KEY__configuration: inputs[KEY__configuration]
+            KEY__configuration: inputs[KEY__configuration],
+            KEY__tenant: tenant,
+            KEY__application: inputs[KEY__application]
         }, as_objects=True)
-        app_url = template.pop(KEY__url)
+        app_url = template.pop(KEY__application_url)
+        artifact_url = template.pop(KEY__artifact_base_uri)
 
         params = inputs[KEY__parameters]
         if params is not None and template[KEY__data_validation_table] is None:
@@ -570,18 +643,30 @@ class JAAQLModel(BaseJAAQLModel):
         if template[KEY__data_validation_table] is not None and params is None:
             params = {}
         if template[KEY__data_validation_table] is not None:
-            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], inputs[KEY__configuration],
-                                                        template[KEY__schema])
+            if EMAIL_PARAM__app_url in params:
+                raise HttpStatusException(ERR__unexpected_validation_column % EMAIL_PARAM__app_url)
+
+            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], inputs[KEY__configuration], template[KEY__schema])
             params, _ = self.fetch_sanitized_email_params(interface, template, params)
 
-        recipient_names = inputs[KEY__recipient]
+        optional_params = {EMAIL_PARAM__app_url: app_url}
+        optional_params = {**optional_params, **params}
+
         recipients = inputs[KEY__recipient]
+        if recipients is None:
+            recipients = username
 
-        self.email_manager.construct_and_send_email(self.url, app_url, template, user_id, recipient_names, recipients, {}, params,
-                                                    attachments=EmailAttachment.deserialize_list(inputs[KEY__attachments], template[KEY__id]),
-                                                    attachment_access_token=self.refresh_auth_token(oauth_token, ip_address))
+        if inputs[KEY__attachments] is not None:
+            for attachment in inputs[KEY__attachments]:
+                attachment[KEY__application] = inputs[KEY__application]
+                attachment[KEY__configuration] = inputs[KEY__configuration]
 
-    def reset_password_perform_reset(self, inputs: dict, tenant: str):
+        self.email_manager.construct_and_send_email(artifact_url, template, user_id, tenant, recipients, None, {}, optional_params,
+                                                    attachments=EmailAttachment.deserialize_list(inputs[KEY__attachments], template[KEY__name],
+                                                                                                 app_url),
+                                                    attachment_access_token=self.refresh_auth_token(auth_token, ip_address))
+
+    def reset_password_perform_reset(self, inputs: dict):
         token = inputs[KEY__reset_key]
         password = inputs[KEY__password]
 
@@ -596,8 +681,11 @@ class JAAQLModel(BaseJAAQLModel):
         }
         if resp[KEY__email_template] and resp[ATTR__data_lookup_json]:
             template = execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_email_template_with_app_url,
-                                                            {KEY__name: resp[KEY__email_template], KEY__tenant: tenant}, as_objects=True)
-            interface = self.fetch_interface_for_tenant(tenant, template[KEY__application], resp[KEY__schema], template[KEY__schema])
+                                                            {KEY__name: resp[KEY__email_template], KEY__tenant: resp[KEY__tenant],
+                                                             KEY__application: resp[KEY__application], KEY__configuration: resp[KEY__configuration]},
+                                                            as_objects=True)
+            interface = self.fetch_interface_for_tenant(resp[KEY__tenant], template[KEY__application], resp[KEY__configuration],
+                                                        template[KEY__schema])
             res[KEY__parameters] = self.select_from_data_validation_table(interface, template[KEY__data_validation_table],
                                                                           json.loads(resp[ATTR__data_lookup_json]))
 
@@ -607,11 +695,10 @@ class JAAQLModel(BaseJAAQLModel):
         return execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__fetch_public_account_credentials,
                                                     {KEY__application: application, KEY__configuration: configuration, KEY__tenant: tenant},
                                                     as_objects=True, encryption_key=self.get_db_crypt_key(),
-                                                    decrypt_columns=[KEY__public_password, KEY__username])
+                                                    decrypt_columns=[KEY__password, KEY__username])
 
     def create_tenant_account(self, connection: DBInterface, tenant: str, inputs: dict, specifying_tenant: bool = False):
-        is_public = inputs.get(KEY__application) is not None
-        if is_public:
+        if inputs.get(KEY__password) is not None:
             crypt_utils.validate_password(inputs[KEY__password])
 
         if KEY__application not in inputs:
@@ -620,14 +707,18 @@ class JAAQLModel(BaseJAAQLModel):
             inputs[KEY__configuration] = None
         if KEY__password not in inputs:
             inputs[KEY__password] = None
-
+        if inputs[KEY__password] is None and inputs[KEY__application] is not None:
+            inputs[KEY__password] = str(uuid.uuid4())  # Set a default password for public user
+        the_password = inputs[KEY__password]
         query = QUERY__create_account_for_tenant if specifying_tenant else QUERY__create_tenant_account
+        if specifying_tenant:
+            inputs[KEY__tenant] = tenant
         account_id = execute_supplied_statement_singleton(connection, query, inputs,
                                                           encryption_key=self.get_db_crypt_key(), encrypt_parameters=[KEY__username, KEY__password],
                                                           encryption_salts={KEY__username: self.get_repeatable_salt(tenant)})[RET__rows][0]
 
-        if inputs.get(inputs[KEY__password]):
-            self.add_account_password(account_id, inputs[KEY__password])
+        if the_password:
+            self.add_account_password(account_id, the_password)
 
         return account_id
 
@@ -668,12 +759,18 @@ class JAAQLModel(BaseJAAQLModel):
             conn = self.jaaql_lookup_connection.get_conn()
             self.jaaql_lookup_connection.execute_script_file(conn, join(get_jaaql_root(), DIR__scripts, "install_2.sql"))
             self.jaaql_lookup_connection.commit(conn)
-
-            resp = self.jaaql_lookup_connection.execute_query(conn, QUERY__setup_system)
-            tenant_admin_account_id = resp[1][0][0]
-            self.jaaql_lookup_connection.commit(conn)
             self.jaaql_lookup_connection.put_conn(conn)
 
+            _, jesk = crypt_utils.key_stretcher(str(uuid.uuid4()), length=crypt_utils.AES__key_length)
+            jesk = jesk.decode()
+            _, desk = crypt_utils.key_stretcher(str(uuid.uuid4()), length=crypt_utils.AES__key_length)
+            desk = desk.decode()
+            resp = execute_supplied_statement(self.jaaql_lookup_connection, QUERY__setup_system,
+                                              {KEY__jaaql_enc_symmetric_key: jesk, KEY__default_enc_symmetric_key: desk},
+                                              encrypt_parameters=[KEY__jaaql_enc_symmetric_key, KEY__default_enc_symmetric_key],
+                                              encryption_key=self.get_db_crypt_key())
+            tenant_admin_account_id = resp[RET__rows][0][0]
+
             execute_supplied_statement_singleton(self.jaaql_lookup_connection, QUERY__attach_account, {
                 KEY__the_tenant: TENANT__jaaql,
                 KEY__user_id: ROLE__jaaql,
@@ -706,6 +803,29 @@ class JAAQLModel(BaseJAAQLModel):
         else:
             raise HttpStatusException(ERR__already_installed)
 
+    def verification_thread(self, the_queue: Queue):
+        print("Starting auth verification thread")
+        while True:
+            auth_token, ip_address, complete = the_queue.get()
+            try:
+                self.verify_auth_token(auth_token, ip_address)
+                complete.put((True, None, None))
+            except HttpStatusException as ex:
+                complete.put((False, ex.message, ex.response_code))
+            except Exception as ex:
+                complete.put((False, str(ex), 500))
+
+    def verify_auth_token_threaded(self, auth_token: str, ip_address: str, complete: Queue):
+        try:
+            if JAAQLModel.VERIFICATION_QUEUE is None:
+                JAAQLModel.VERIFICATION_QUEUE = Queue()
+                threading.Thread(target=self.verification_thread, args=[JAAQLModel.VERIFICATION_QUEUE], daemon=True).start()
+            JAAQLModel.VERIFICATION_QUEUE.put((auth_token, ip_address, complete))
+            payload = json.loads(base64url_decode(auth_token.split(".")[1].encode("UTF-8")).decode())
+            return payload[KEY__user_id], payload[KEY__tenant], payload[KEY__username], payload[KEY__ip_id], payload[KEY__is_public]
+        except Exception:
+            raise HttpStatusException(ERR__invalid_token, HTTPStatus.UNAUTHORIZED)
+
     def verify_auth_token(self, auth_token: str, ip_address: str):
         decoded = crypt_utils.jwt_decode(self.vault.get_obj(VAULT_KEY__jwt_crypt_key), auth_token, JWT_PURPOSE__oauth)
         if not decoded or decoded[KEY__address] != ip_address:
@@ -727,13 +847,10 @@ class JAAQLModel(BaseJAAQLModel):
         return self.get_auth_token(decoded[KEY__username], decoded[KEY__tenant], ip_address)
 
     def get_auth_token(self, username: str, tenant: str, ip_address: str, password: str = None, response: JAAQLResponse = None):
-        hash_username = username
-        if len(hash_username) < 8:
-            hash_username = hash_username * 8
-            hash_username = hash_username[0:8]
-        salt_tenant_username = self.get_repeatable_salt(tenant + hash_username)
+        user_id = self.fetch_user_from_username(username, tenant)
+        salt_tenant_user = self.get_repeatable_salt(tenant + user_id)
 
-        encrypted_ip = jaaql__encrypt(ip_address, self.get_db_crypt_key(), salt_tenant_username)
+        encrypted_ip = jaaql__encrypt(ip_address, self.get_db_crypt_key(), salt_tenant_user)
 
         the_kwargs = {
             "encryption_salts": {KEY__enc_username: self.get_repeatable_salt(tenant)},
@@ -788,19 +905,22 @@ class JAAQLModel(BaseJAAQLModel):
         return crypt_utils.jwt_encode(self.vault.get_obj(VAULT_KEY__jwt_crypt_key), jwt_data, JWT_PURPOSE__oauth, expiry_ms=self.token_expiry_ms)
 
     def fetch_pivoted_application_schemas(self, tenant: str):
-        data = execute_supplied_statement(self.jaaql_lookup_connection, QUERY__fetch_application_schemas, {KEY__the_tenant: tenant}, as_objects=True)
+        data = execute_supplied_statement(self.jaaql_lookup_connection, QUERY__fetch_application_schemas, {KEY__tenant: tenant}, as_objects=True,
+                                          skip_commit=True)
 
         return self.group(self.pivot(data,
                           [JAAQLPivotInfo(KEY__configuration, "name", "configuration_name"),
                            JAAQLPivotInfo(KEY__schema, "configuration_name", "schema_name")]), KEY__name)
 
-    def create_interface_for_db(self, user_id: str, database: str):
+    def create_interface_for_db(self, user_id: str, database: str, sub_role: str = None):
         jaaql_uri = self.vault.get_obj(VAULT_KEY__super_db_credentials)
         address, port, _, username, password = DBInterface.fracture_uri(jaaql_uri)
-        return create_interface(self.config, address, port, database, username, password=password, role=user_id)
+        return create_interface(self.config, address, port, database, username, password=password, role=user_id, sub_role=sub_role)
 
-    def submit(self, inputs: dict, user_id: str, tenant: str, force_transactional: bool = False):
+    def submit(self, inputs: dict, user_id: str, tenant: str, verification_hook: Queue):
+        was_dict = False
         if isinstance(inputs, dict):
+            was_dict = True
             inputs = [inputs]
 
         tenant_application_configs = None
@@ -820,7 +940,7 @@ class JAAQLModel(BaseJAAQLModel):
                     cur.pop(KEY__schema)
                 else:
                     if len(config) == 1:
-                        found_db = config[config.keys[0]][KEY__database]
+                        found_db = config[list(config.keys())[0]][KEY__database]
                     else:
                         found_db = [val[KEY__database] for _, val in config.items() if val[KEY__is_default]]
                         if len(found_db) == 1:
@@ -837,6 +957,19 @@ class JAAQLModel(BaseJAAQLModel):
                 cur[KEY__database] = tenant + "__" + cur[KEY__database]
 
             if cur[KEY__database] not in required_dbs:
-                required_dbs[cur[KEY__database]] = self.create_interface_for_db(user_id, cur[KEY__database])
-
-        return [InterpretJAAQL(required_dbs[cur.pop(KEY__database)]).transform(inputs, force_transactional=force_transactional) for cur in inputs]
+                # Technical debt. If we switch roles here for the first query, it'll be shared amongst others. Not a security risk as the other must
+                # have access to the role to switch to it
+                sub_role = cur.pop(KEY__role) if KEY__role in cur else None
+                if sub_role is not None:
+                    sub_role = tenant + "__" + sub_role
+                required_dbs[cur[KEY__database]] = self.create_interface_for_db(user_id, cur[KEY__database], sub_role)
+        res = []
+        for cur, idx in zip(inputs, range(len(inputs))):
+            pass_hook = verification_hook if idx == 0 else None
+            ret = InterpretJAAQL(required_dbs[cur.pop(KEY__database)]).transform(inputs, skip_commit=cur.get(KEY__read_only), wait_hook=pass_hook,
+                                                                                 encryption_key=self.fetch_tenant_symmetric_key(tenant))[0]
+            res.append(ret)
+        if was_dict:
+            return res[0]
+        else:
+            return res
